<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/classKGatewayNetwork.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';

    $AjaxFunctions['showNetworkGW'] = 'showNetworkGW';
    $AjaxFunctions['saveNetworkGW'] = 'saveNetworkGW';
    $AjaxFunctions['showVoIPOptionsGW'] = 'showVoIPOptionsGW';
    $AjaxFunctions['showVoIPTLSGW'] = 'showVoIPTLSGW';
    $AjaxFunctions['getHostName'] = 'getHostName';

    define( 'GATEWAY_NETWORK_CONFIG', '/etc/network/interfaces' );

    class KGatewayNetworkConfigException extends Exception
    {
        /* VOID */
    }

    function iseth( $intf )
    {
        // GAMBI: return ONLY ethX interfaces, rá!
        return strpos( $intf, 'eth' ) === 0 && strlen( $intf ) === 4;
    }

    function showNetworkGW( )
    {
        require_once KCONFIG . '/classes/KGatewayHardwareInfo.php';

        $app = KSystem::getInstance( );
        $hwinfo = KGatewayHardwareInfo::getInstance( );
        $parser = new KGatewayNetwork( GATEWAY_NETWORK_CONFIG );
        $parser->load( );

        $buffer = '';

        $buffer .= '<div class="textAlignCenter displayNone sms-info" id="gateway-reboot-is-needed">';
        $buffer .= "<label>{$app->translate( 'gateway-network-alert-reboot' )}</label>";

        $buffer .= '<br /><br />';

        $buffer .= "<input id='gateway-alert-reboot-bt' type='button' onclick='rebootGW();' value='{$app->translate( 'gateway-reboot' )}' />";
        $buffer .= "</div>";

        $buffer .= '<br />';

        $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network' )}</legend>";
        $buffer .= "<div id='gateway-network'>";

        $switchcores = count( $hwinfo->query( 'Switch.Cores' ) );

        if( $switchcores > 0 )
        {
            $buffer .= showNetworkVlan( $parser, $hwinfo );
        }
        else
        {
            $buffer .= showNetworkNoVlan( $parser );
        }

        // DNS SERVERS
        $servers = $parser->getDNSServers( );

        $buffer .= '<div id=\'gateway-network-dns\'><fieldset><legend>DNS</legend>';

        if( $switchcores > 0 )
        {
            $buffer .= '<input type=\'hidden\' id=\'gateway-network-dns-method\' value=\'static\' />';
        }
        else
        {
            $buffer .= "<div class='gateway-network-method'><label for='gateway-network-dns-method'>{$app->translate( 'gateway-network-dns-method' )}</label> <select id='gateway-network-dns-method'>";

            foreach( KGatewayNetwork::$dnsmethods as $method )
            {
                $selected = (!empty( $servers ) && $method == 'static') ? ' selected=\'true\'' : '';
                $name = $app->translate( "gateway-network-method-{$method}" );
                $buffer .= "<option value='{$method}'{$selected}>{$name}</option>";
            }

            $buffer .= '</select></div><br />';
        }

        $dnssearch = $parser->getDNSSearchDomain( );

        $buffer .= '<div id=\'gateway-network-dns-config\'><table class=\'gateway-table-form zebra simpleTable\'><tbody>';
        $buffer .= "<tr><td class='label'><label for='gateway-network-dns-search'>{$app->translate( 'gateway-network-dns-search' )}</label></td><td class='gateway-form-field-container-right'><input type='text' id='gateway-network-dns-search' value='{$dnssearch}' /></td></tr>";
        $buffer .= "<tr><td class='label'>{$app->translate( 'gateway-network-servers' )}</td><td class='gateway-form-field-container-right'>";
        for( $i = 0; $i < 3; ++$i )
        {
            $value = isset( $servers[$i] ) ? $servers[$i] : '';
            $buffer .= "<input type='text' class='gateway-network-dns-server' value='{$value}' /><br />";
        }
        $buffer .= "</td></tr>";
        $buffer .= '</tbody></table></fieldset></div><br />';
        unset( $servers );
        unset( $dnssearch );

        $buffer .= "<br /><div class='textAlignRight'><input type='button' id='gateway-network-save' value='{$app->translate( 'save' )}' /></div>";

        return "{$buffer}</div></fieldset>";
    }

    function __networkMethodSelect( $intf, $method )
    {
        $app = KSystem::getInstance( );

        $buf = "<select id='gateway-network-method-{$intf}' class='gateway-network-interface-method'>";
        foreach( KGatewayNetwork::$methods['inet'] as $option )
        {
            $selected = $method === $option ? ' selected=\'selected\'' : '';

            $name = $app->translate( "gateway-network-method-{$option}" );
            $buf .= "<option value='{$option}'{$selected}>{$name}</option>";
        }
        return "{$buf}</select>";
    }

    function __networkVLANModeCheckbox( $intf, $mode )
    {
        $app = KSystem::getInstance( );
        $checked = ( $mode === 'MEMBER' ) ? " checked='checked'" : '';
        return "<input id='gateway-network-vlan-mode-{$intf}' type='checkbox' value='true' class='vlan-mode'{$checked}/>";
    }

    function __networkInput( $intf, $name, $value, $class = '' )
    {
        return "<input type='text' value='{$value}' id='gateway-network-{$name}-{$intf}' class='{$class}' />";
    }

    function showNetworkVlan( KGatewayNetwork $parser, KGatewayHardwareInfo $hwinfo )
    {
        require_once KCONFIG . '/classes/KGatewaySwitch.php';

        $app = KSystem::getInstance( );
        $cores = $hwinfo->query( 'Switch.Cores' );
        $switchyaml = new KGatewaySwitchYaml( );

        $buf = '<div id="gateway-network-vlan-interfaces"><input type="hidden" value="vlan" id="gateway-network-configuration-mode" />';

        $buf .= "<table class='gateway-table-form zebra simpleTable'><thead><tr><th rowspan='2'>{$app->translate( 'gateway-network-interface' )}</th><th rowspan='2'>{$app->translate( 'gateway-network-method' )}</th><th colspan='3'>IPv4</th><th colspan='3'>IPv6</th><th rowspan='2' class='advanced-options'>VLAN</th></tr>";
        $buf .= "<tr><th>{$app->translate( 'gateway-network-address' )}</th><th>{$app->translate( 'gateway-network-netmask' )}</th><th>{$app->translate( 'gateway-network-gateway' )}</th><th>{$app->translate( 'gateway-network-address' )}</th><th>{$app->translate( 'gateway-network-netmask' )}</th><th>{$app->translate( 'gateway-network-gateway' )}</th></tr><tbody>";

        foreach( $hwinfo->getSwitchPortNames() as $idx => $name )
        {
            $port = $hwinfo->getSwitchPortConfig( $name );
            $swconfig = $switchyaml->getVLANConfigByPortName( $name );
            if( is_null( $port ) || is_null( $swconfig ) )
            {
                return $app->translate( 'gateway-network-interface-error-invalid-config' );
            }

            $intf = "{$cores[$port['CoreId']]['Interface']}";
            $vlanintf = "{$intf}.{$swconfig['Id']}";
            $config = $parser->hasInterface( $vlanintf ) ? $parser->getConfiguration( $vlanintf ) : array( );

            $method = isset( $config['inet']['properties']['method'] ) ? $config['inet']['properties']['method'] : 'manual';

            $ipv4address = isset( $config['inet']['config']['address'] ) ? $config['inet']['config']['address'] : '';
            $ipv4netmask = isset( $config['inet']['config']['netmask'] ) ? $config['inet']['config']['netmask'] : '';
            $ipv4gateway = isset( $config['inet']['config']['gateway'] ) ? $config['inet']['config']['gateway'] : '';

            $ipv6address = isset( $config['inet6']['config']['address'] ) ? $config['inet6']['config']['address'] : '';
            $ipv6netmask = isset( $config['inet6']['config']['netmask'] ) ? $config['inet6']['config']['netmask'] : '';
            $ipv6gateway = isset( $config['inet6']['config']['gateway'] ) ? $config['inet6']['config']['gateway'] : '';

            $methodselect = __networkMethodSelect( $name, $method, true );
            $vlanmodecheckbox = __networkVLANModeCheckbox( $name, $swconfig['Mode'] );

            $ipv4addressinput = __networkInput( $name, 'ipv4-address', $ipv4address, "interface-{$name}" );
            $ipv4netmaskinput = __networkInput( $name, 'ipv4-netmask', $ipv4netmask, "interface-{$name}" );
            $ipv4gatewayinput = __networkInput( $name, 'ipv4-gateway', $ipv4gateway, "interface-{$name}" );

            $ipv6addressinput = __networkInput( $name, 'ipv6-address', $ipv6address, "interface-{$name}" );
            $ipv6netmaskinput = __networkInput( $name, 'ipv6-netmask', $ipv6netmask, "interface-{$name}" );
            $ipv6gatewayinput = __networkInput( $name, 'ipv6-gateway', $ipv6gateway, "interface-{$name}" );

            $vlaninput = __networkInput( $name, 'vlan', $swconfig['Id'], 'vlan' );

            $buf .= "<tr><td>{$name}<input type='hidden' class='gateway-network-interface' value='{$name}' /></td>";
            $buf .= "<td class='border-right'>{$methodselect}</td>";
            $buf .= "<td>{$ipv4addressinput}</td>";
            $buf .= "<td>{$ipv4netmaskinput}</td>";
            $buf .= "<td class='border-right'>{$ipv4gatewayinput}</td>";
            $buf .= "<td>{$ipv6addressinput}</td>";
            $buf .= "<td>{$ipv6netmaskinput}</td>";
            $buf .= "<td class='border-right'>{$ipv6gatewayinput}</td>";
            $buf .= "<td class='advanced-options'>{$vlanmodecheckbox} {$vlaninput}</td>";

            $buf .= '</tr>';
        }
        return "{$buf}</tbody></table></div><div class='floatRight'><span id='gateway-network-advanced-options' class='link'>{$app->translate( 'gateway-network-advanced-options' )}</span></div><br />";
    }

    function showNetworkNoVlan( KGatewayNetwork $parser )
    {
        $app = KSystem::getInstance( );

        $buffer = '<input type="hidden" value="no-vlan" id="gateway-network-configuration-mode" />';
        $ifaces = array_filter( $parser->getInterfaces( ), 'iseth' );
        $total_interfaces = count( $ifaces );

        try
        {
            foreach( $ifaces as $idx => $iface )
            {
                try
                {
                    $config = $parser->getConfiguration( $iface );
                    $macaddress = $parser->getMacAddress( $iface );
                }
                catch( Exception $e )
                {
                    if( $total_interfaces > 1 )
                    {
                        trigger_error( "{$iface}: {$e->getMessage()}", E_USER_NOTICE );
                        continue;
                    }
                    throw $e;
                }

                $inet = array( "address" => array_key_exists( 'address', $config['inet']['config'] ) ? $config['inet']['config']['address'] : '', "netmask" => array_key_exists( 'netmask', $config['inet']['config'] ) ? $config['inet']['config']['netmask'] : '', "gateway" => array_key_exists( 'gateway', $config['inet']['config'] ) ? $config['inet']['config']['gateway'] : '' );

                $inet6 = array( "address" => array_key_exists( 'address', $config['inet6']['config'] ) ? $config['inet6']['config']['address'] : '', "netmask" => array_key_exists( 'netmask', $config['inet6']['config'] ) ? $config['inet6']['config']['netmask'] : '', "gateway" => array_key_exists( 'gateway', $config['inet6']['config'] ) ? $config['inet6']['config']['gateway'] : '' );

                $buffer .= "<div id='gateway-network-{$iface}' class='gateway-network-interface'>";

                if( $total_interfaces > 1 )
                {
                    $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network-interface' )} {$iface} [{$app->translate( 'gateway-network-interface-macaddress' )} {$macaddress}]</legend>";
                }
                else
                {
                    $buffer .= "<fieldset><legend>{$app->translate( 'gateway-network-interface' )} [{$app->translate( 'gateway-network-interface-macaddress' )} {$macaddress}]</legend>";
                }

                $method = 'dhcp';
                if( isset( $config['inet']['properties']['method'] ) )
                {
                    $method = $config['inet']['properties']['method'];
                }
                else if( isset( $config['inet6']['properties']['method'] ) )
                {
                    $method = $config['inet6']['properties']['method'];
                }

                $buffer .= "<div class='gateway-network-method'><label for='gateway-network-method-{$iface}'>{$app->translate( 'gateway-network-method' )}</label>";
                $buffer .= __networkMethodSelect( $iface, $method );
                $buffer .= '</div><br />';
                $buffer .= "<div id='gateway-network-config-dhcp-{$iface}' class='displayNone'>";
                $buffer .= '<table><tbody>';
                $buffer .= '</tbody></table>';
                $buffer .= "</div>";
                $buffer .= "<div id='gateway-network-config-static-{$iface}' class='displayNone'>";
                $buffer .= "<table class='gateway-table-form zebra ipv4'>";
                $buffer .= "<caption>IPv4</caption>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-address-{$iface}'>{$app->translate( 'gateway-network-address' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-address-{$iface}' class='ipv4' type='text' value='{$inet['address']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-netmask-{$iface}'>{$app->translate( 'gateway-network-netmask' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-netmask-{$iface}' class='ipv4' type='text' value='{$inet['netmask']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-gateway-{$iface}'>{$app->translate( 'gateway-network-gateway' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-gateway-{$iface}' class='ipv4' type='text' value='{$inet['gateway']}' /></td></tr>";
                $buffer .= "</table>";
                $buffer .= "<br />";
                $buffer .= "<table class='gateway-table-form zebra ipv6'>";
                $buffer .= "<caption>IPv6</caption>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-addressv6-{$iface}'>{$app->translate( 'gateway-network-address' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-addressv6-{$iface}' class='ipv6' type='text' maxlength='39' value='{$inet6['address']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-netmaskv6-{$iface}'>{$app->translate( 'gateway-network-netmask' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-netmaskv6-{$iface}' type='text' size='3' maxlength='3' value='{$inet6['netmask']}' /></td></tr>";
                $buffer .= "<tr><td class='label'><label for='gateway-network-static-gatewayv6-{$iface}'>{$app->translate( 'gateway-network-gateway' )}</label></td><td class='gateway-form-field-container-right'><input id='gateway-network-static-gatewayv6-{$iface}' class='ipv6' type='text' maxlength='39' value='{$inet6['gateway']}' /></td></tr>";
                $buffer .= "</table>";
                $buffer .= "</div>";

                unset( $inet );
                unset( $inet6 );

                $buffer .= "</fieldset><br />";
            }
        }
        catch( KGatewayNetworkException $e )
        {
            $buffer .= $app->translate( $e->getMessage( ) );
        }

        return $buffer;
    }

    function saveNetworkGW( )
    {
        require_once KROOT . '/json_util.php';
        require_once KCONFIG . '/classes/KGatewayHardwareInfo.php';

        $app = KSystem::getInstance( );
        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-network-error-save' ) );
        $hwinfo = KGatewayHardwareInfo::getInstance( );

        try
        {
            $switchcores = count( $hwinfo->query( 'Switch.Cores' ) );

            if( $switchcores > 0 )
            {
                saveNetworkVlan( $hwinfo );
            }
            else
            {
                saveNetworkNoVlan( $hwinfo );
            }

            $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-network-saved' ) );
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }
        return JSONencode( $reply );
    }

    function saveNetworkVlan( KGatewayHardwareInfo $hwinfo )
    {
        require_once KCONFIG . '/classes/KGatewaySwitch.php';

        $parser = new KGatewayNetwork( GATEWAY_NETWORK_CONFIG );
        $parser->load();
        $switchyaml = new KGatewaySwitchYaml();
        $cores = $hwinfo->query( 'Switch.Cores' );

        // build list of current vlan interfaces that should be configured, so
        // we can remove the obsolete/renamed interfaces at the end
        $removeintf = array( );
        foreach( $hwinfo->getSwitchPortNames() as $idx => $name )
        {
            $port = $hwinfo->getSwitchPortConfig( $name );
            $swconfig = $switchyaml->getVLANConfigByPortName( $name );
            if( is_null( $port ) || is_null( $swconfig ) )
            {
                throw new KGatewayNetworkConfigException( 'gateway-network-error-invalid-interface' );
            }

            $intf = "{$cores[$port['CoreId']]['Interface']}";
            $removeintf[] = "{$intf}.{$swconfig['Id']}";
        }

        // update configuration/add interfaces
        foreach( $_POST as $name => &$config )
        {
            $port = $hwinfo->getSwitchPortConfig( $name );
            $core = $cores[$port['CoreId']];
            $intf = $core['Interface'];
            $vlanintf = "{$intf}.{$config['vlan']['id']}";

            if( ! empty( $config['iface']['inet']['properties'] ) )
            {
                $parser->setProperties( $vlanintf, $config['iface']['inet']['properties'], $config['iface']['inet']['config'] );

                if ( !empty( $config[ 'iface' ][ 'inet' ][ 'config' ] ) )
                {
                    $parser->setConfiguration( $vlanintf, $config[ 'iface' ][ 'inet' ][ 'config' ], $config[ 'iface' ][ 'inet' ][ 'properties' ][ 'method' ], $config[ 'iface' ][ 'inet' ][ 'properties' ][ 'family' ]);
                }
            }
            if( ! empty( $config['iface']['inet6']['properties'] ) )
            {
                $parser->setProperties( $vlanintf, $config['iface']['inet6']['properties'], $config['iface']['inet6']['config'] );

                if ( !empty( $config[ 'iface' ][ 'inet6' ][ 'config' ] ) )
                {
                    $parser->setConfiguration( $vlanintf, $config[ 'iface' ][ 'inet6' ][ 'config' ], $config[ 'iface' ][ 'inet6' ][ 'properties' ][ 'method' ], $config[ 'iface' ][ 'inet6' ][ 'properties' ][ 'family' ]);
                }
            }
            $switchyaml->setVLANConfig( $name, $config['vlan']['id'], $config['vlan']['mode'] );

            $pos = array_search( $vlanintf, $removeintf );
            if( $pos !== false )
            {
                unset( $removeintf[$pos] );
            }
        }

        // remove obsolete/renamed interfaces
        foreach( $removeintf as $intf )
        {
            $parser->removeInterface( $intf );
        }

        $switchyaml->save( );
        $parser->save( );
    }

    function saveNetworkNoVlan( KGatewayHardwareInfo $hwinfo )
    {
        $parser = new KGatewayNetwork( GATEWAY_NETWORK_CONFIG );
        // interfaces configuration
        foreach( $_POST['ifaces'] as $iface => &$familyData )
        {
            foreach( $familyData as $family => &$data )
            {
                $config = array_key_exists( 'config', $data ) ? $data['config'] : array( );
                if( empty( $data['properties'] ) )
                {
                    continue;
                }

                $parser->setProperties( $iface, $data['properties'], $config );

                if ( !empty( $config ) ) 
                {
                    $parser->setConfiguration( $iface,  $config, $data[ 'properties' ][ 'method' ], $data[ 'properties' ][ 'family' ] );
                }
            }
        }

        $parser->save( );
    }

    function showVoIPOptionsGW( $_REQUEST )
    {
        return showVoIPOptions( $_REQUEST, 'GW' );
    }

    function showVoIPTLSGW( $_REQUEST )
    {
        return showVoIPTLS( $_REQUEST, 'GW' );
    }

    function getHostName( )
    {
        $hostname = exec( "hostname -s" );
        return JSONencode( $hostname );
    }
